//
//  bookshelfViewController.m
//  Bookshelf
//
//  Created by Fred Wu on 7/13/15.
//  Copyright (c) 2015 Fred. All rights reserved.
//

#import "BookshelfViewController.h"
#import "BookCell.h"
#import "BookHeaderView.h"
#import "BookFooterView.h"

@interface BookshelfViewController ()<UITextFieldDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout>{
    NSMutableArray *books;
    unsigned booksCount;
    NSCache *imageCache;
}


@end

@implementation BookshelfViewController

static NSString * const reuseIdentifier = @"Book Cover Cell";
static unsigned const NUMBER_PER_SECTION = 2;

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.collectionView.contentMode = UIViewContentModeScaleToFill;
    self.collectionView.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"bookshelf-bg-375"]];


    imageCache = [[NSCache alloc] init];
    [self.collectionView registerNib:[UINib nibWithNibName:@"BookCell" bundle:nil]
          forCellWithReuseIdentifier:@"Book Cover Cell"];
    [self.collectionView registerClass:[BookHeaderView class] forSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"Book Header"];
    [self.collectionView registerClass:[BookFooterView class] forSupplementaryViewOfKind:UICollectionElementKindSectionFooter withReuseIdentifier:@"Book Footer"];
    [self loadBooks];
    // Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


- (BOOL)loadBooks {
    
    NSString * filePath =[[NSBundle mainBundle] pathForResource:@"sample" ofType:@"json"];
    NSData *data = [[NSData alloc] initWithContentsOfFile:filePath];
    NSError *error=nil;
    books = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:&error];
    
    booksCount = (unsigned)[books count];
    
    if(error)
    {
        NSLog(@"Error reading file: %@",error.localizedDescription);
        return NO;
    }else {
        return YES;
    }
    
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

#pragma mark <UICollectionViewDataSource>

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return booksCount / NUMBER_PER_SECTION;
}


- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if (section < booksCount / NUMBER_PER_SECTION) {
        return NUMBER_PER_SECTION;
    }else{
        return booksCount - NUMBER_PER_SECTION * section;
    }
    return 0;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    return CGSizeMake(100, 170);
}

- (UIEdgeInsets)collectionView: (UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout insetForSectionAtIndex:(NSInteger)section {
    return UIEdgeInsetsMake(0, 60, 0, 60);
}


- (BookCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    
    
    long bookIndex = indexPath.section * NUMBER_PER_SECTION + indexPath.row;
    BookCell *cell = (BookCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"Book Cover Cell" forIndexPath:indexPath];
    NSDictionary *book = [books objectAtIndex:bookIndex];
    NSURL *imageURL = [NSURL URLWithString:book[@"url"]];
    
    UIImage *image = [imageCache objectForKey:imageURL];
    if(image){
        cell.imageView.image = image;
    }else{
        cell.imageView.image = nil;
        dispatch_queue_t downloadQueue = dispatch_queue_create("image downloader", NULL);
        dispatch_async(downloadQueue, ^{
            
            NSData *data = [NSData dataWithContentsOfURL:imageURL];
            UIImage *image = [UIImage imageWithData:data];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                cell.imageView.image = image;
            });
            [imageCache setObject:image forKey:imageURL];
        });
    }
    
    NSString *title = [NSString stringWithFormat:@"%@",book[@"title"]];
    
    cell.bookTitleLabel.text = title;
    cell.bookTitleLabel.textColor =  [[UIColor blackColor] colorWithAlphaComponent:0.5f];
    
    cell.imageView.layer.shadowColor = [UIColor blackColor].CGColor;
    cell.imageView.layer.shadowOffset = CGSizeMake(1, 1);
    cell.imageView.layer.shadowOpacity = 0.5;
    cell.imageView.layer.shadowRadius = 3.0;
    cell.imageView.clipsToBounds = NO;
    
    CALayer *layer = cell.imageView.layer;
    CATransform3D rotationAndPerspectiveTransform = CATransform3DIdentity;
    rotationAndPerspectiveTransform.m34 = 1.0 / -500;
    rotationAndPerspectiveTransform = CATransform3DRotate(rotationAndPerspectiveTransform, 10.0f * M_PI / 180.0f, 1.0f, 0.0f, 0.0f);
    layer.transform = rotationAndPerspectiveTransform;
//    cell.bookTitleLabel.textColor = [UIColor whiteColor];
    cell.backgroundColor = nil;
    
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section{
    if (section == 0){
        return CGSizeMake(self.collectionView.frame.size.width, 50);
    }else{
        return CGSizeMake(0, 0);
    }
}


- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout referenceSizeForFooterInSection:(NSInteger)section{
    return CGSizeMake(self.collectionView.frame.size.width, 34);
}


- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath{
    UICollectionReusableView *reusableview = nil;
    
    if (kind == UICollectionElementKindSectionHeader) {
        BookHeaderView *v = [collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"Book Header" forIndexPath:indexPath];
        reusableview = v;
    }
    
    if (kind == UICollectionElementKindSectionFooter) {
        BookFooterView *v = [collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionFooter withReuseIdentifier:@"Book Footer" forIndexPath:indexPath];
        reusableview = v;
    }
    
    return reusableview;
}

#pragma mark <UICollectionViewDelegate>

/*
// Uncomment this method to specify if the specified item should be highlighted during tracking
- (BOOL)collectionView:(UICollectionView *)collectionView shouldHighlightItemAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}
*/

/*
// Uncomment this method to specify if the specified item should be selected
- (BOOL)collectionView:(UICollectionView *)collectionView shouldSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}
*/

/*
// Uncomment these methods to specify if an action menu should be displayed for the specified item, and react to actions performed on the item
- (BOOL)collectionView:(UICollectionView *)collectionView shouldShowMenuForItemAtIndexPath:(NSIndexPath *)indexPath {
	return NO;
}

- (BOOL)collectionView:(UICollectionView *)collectionView canPerformAction:(SEL)action forItemAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
	return NO;
}

- (void)collectionView:(UICollectionView *)collectionView performAction:(SEL)action forItemAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
	
}
*/

@end
